@@ -31,8 +31,8 @@ namespace xt
     {
         using storage_type = EC;
         using shape_type = SC;
-        using strides_type = shape_type;
-        using backstrides_type = shape_type;
+        using strides_type = get_strides_t<shape_type>;
+        using backstrides_type = get_strides_t<shape_type>;
         using inner_shape_type = shape_type;
         using inner_strides_type = strides_type;
         using inner_backstrides_type = backstrides_type;
@@ -80,6 +80,7 @@ namespace xt
         using strides_type = typename base_type::strides_type;
         using backstrides_type = typename base_type::backstrides_type;
         using inner_strides_type = typename base_type::inner_strides_type;
+        using inner_backstrides_type = typename base_type::inner_backstrides_type;
         using temporary_type = typename semantic_base::temporary_type;
         using expression_tag = Tag;
 
@@ -108,6 +109,11 @@ namespace xt
         xarray_container(xarray_container&&) = default;
         xarray_container& operator=(xarray_container&&) = default;
 
+        template <std::size_t N>
+        explicit xarray_container(xtensor_container<EC, N, L, Tag>&& rhs);
+        template <std::size_t N>
+        xarray_container& operator=(xtensor_container<EC, N, L, Tag>&& rhs);
+
         template <class E>
         xarray_container(const xexpression<E>& e);
 
@@ -133,8 +139,8 @@ namespace xt
     {
         using storage_type = std::remove_reference_t<EC>;
         using shape_type = SC;
-        using strides_type = shape_type;
-        using backstrides_type = shape_type;
+        using strides_type = get_strides_t<shape_type>;
+        using backstrides_type = get_strides_t<shape_type>;
         using inner_shape_type = shape_type;
         using inner_strides_type = strides_type;
         using inner_backstrides_type = backstrides_type;
@@ -210,7 +216,6 @@ namespace xt
         storage_type& storage_impl() noexcept;
         const storage_type& storage_impl() const noexcept;
 
-
         friend class xcontainer<xarray_adaptor<EC, L, SC, Tag>>;
     };
 
@@ -390,6 +395,29 @@ namespace xt
         return self_type(shape);
     }
 
+    template <class EC, layout_type L, class SC, class Tag>
+    template <std::size_t N>
+    inline xarray_container<EC, L, SC, Tag>::xarray_container(xtensor_container<EC, N, L, Tag>&& rhs)
+        : base_type(inner_shape_type(rhs.shape().cbegin(), rhs.shape().cend()),
+                    inner_strides_type(rhs.strides().cbegin(), rhs.strides().cend()),
+                    inner_backstrides_type(rhs.backstrides().cbegin(), rhs.backstrides().cend()),
+                    std::move(rhs.layout())),
+          m_storage(std::move(rhs.storage()))
+    {
+    }
+
+    template <class EC, layout_type L, class SC, class Tag>
+    template <std::size_t N>
+    inline xarray_container<EC, L, SC, Tag>& xarray_container<EC, L, SC, Tag>::operator=(xtensor_container<EC, N, L, Tag>&& rhs)
+    {
+        this->shape_impl().assign(rhs.shape().cbegin(), rhs.shape().cend());
+        this->strides_impl().assign(rhs.strides().cbegin(), rhs.strides().cend());
+        this->backstrides_impl().assign(rhs.backstrides().cbegin(), rhs.backstrides().cend());
+        this->mutable_layout() = rhs.layout();
+        m_storage = std::move(rhs.storage());
+        return *this;
+    }
+
     /**
      * @name Extended copy semantic
      */
