@@ -25,6 +25,7 @@
 #include <xtl/xsequence.hpp>
 
 #include "xbuilder.hpp"
+#include "xeval.hpp"
 #include "xexpression.hpp"
 #include "xgenerator.hpp"
 #include "xiterable.hpp"
@@ -58,6 +59,9 @@ namespace xt
     template <class ST, class X>
     struct xreducer_shape_type;
 
+    template <class S1, class S2>
+    struct fixed_xreducer_shape_type;
+
     template <class E, class X, class result_type>
     struct xreducer_result_container
     {
@@ -76,14 +80,19 @@ namespace xt
         using type = xtensor<result_type, sizeof...(N) - NX, L>;
     };
 
+    template <class T, std::size_t... I, layout_type L, std::size_t... X, class result_type>
+    struct xreducer_result_container<xtensor_fixed<T, xshape<I...>, L>, xshape<X...>, result_type>
+    {
+        using type = xtensor_fixed<result_type, typename fixed_xreducer_shape_type<fixed_shape<I...>, fixed_shape<X...>>::type, L>;
+    };
+
     template <class F, class E, class X>
-    auto reduce_immediate(F&& f, E&& e, X&& axes)
+    inline auto reduce_immediate(F&& f, E&& e, X&& axes)
     {
         using shape_type = typename xreducer_shape_type<typename std::decay_t<E>::shape_type, std::decay_t<X>>::type;
 
         using reduce_functor_type = typename std::decay_t<F>::reduce_functor_type;
         using init_functor_type = typename std::decay_t<F>::init_functor_type;
-        using merge_functor_type = typename std::decay_t<F>::merge_functor_type;
 
         using expr_value_type = typename std::decay_t<E>::value_type;
 
@@ -95,7 +104,7 @@ namespace xt
         auto init_fct = std::get<1>(f);
         auto merge_fct = std::get<2>(f);
 
-        shape_type result_shape;
+        shape_type result_shape{};
         resize_container(result_shape, e.dimension() - axes.size());
 
         dynamic_shape<std::size_t> iter_shape = xtl::forward_sequence<dynamic_shape<std::size_t>>(e.shape());
@@ -136,10 +145,17 @@ namespace xt
 
         result.resize(result_shape, e.layout());
 
-        std::size_t ax_idx = (e.layout() == layout_type::row_major) ? axes.size() - 1 : 0;
-        std::size_t inner_loop_size = e.strides()[axes[ax_idx]];
-        std::size_t inner_stride = e.strides()[axes[ax_idx]];
-        std::size_t outer_loop_size = e.shape()[axes[ax_idx]];
+        std::size_t leading_ax = axes[(e.layout() == layout_type::row_major) ? axes.size() - 1 : 0];
+        auto strides_finder = e.strides().begin() + static_cast<std::ptrdiff_t>(leading_ax);
+        // The computed strides contain "0" where the shape is 1 -- therefore find the next none-zero number
+        std::size_t inner_stride = static_cast<std::size_t>(*strides_finder);
+        while (inner_stride == 0)
+        {
+            (e.layout() == layout_type::row_major) ? --strides_finder : ++strides_finder;
+            inner_stride = static_cast<std::size_t>(*strides_finder);
+        }
+        std::size_t inner_loop_size = static_cast<std::size_t>(inner_stride);
+        std::size_t outer_loop_size = e.shape()[leading_ax];
 
         // The following code merges reduction axes "at the end" (or the beginning for col_major)
         // together by increasing the size of the outer loop where appropriate
@@ -163,7 +179,7 @@ namespace xt
             if (std::find(axes.begin(), axes.end(), i) == axes.end())
             {
                 // i not in axes!
-                iter_strides[i] = result.strides()[idx];
+                iter_strides[i] = static_cast<std::size_t>(result.strides()[idx]);
                 ++idx;
             }
         }
@@ -267,7 +283,7 @@ namespace xt
             while (idx_res.first != true)
             {
                 std::transform(out, out + inner_loop_size, begin, out,
-                               [merge, &init_fct, &merge_fct, &reduce_fct](auto&& v1, auto&& v2) {
+                               [merge, &init_fct, &reduce_fct](auto&& v1, auto&& v2) {
                                     return merge ?
                                         reduce_fct(v1, v2) :
                                         // cast because return type of identity function is not upcasted
@@ -439,6 +455,8 @@ namespace xt
         const_reference operator()(Args... args) const;
         template <class... Args>
         const_reference at(Args... args) const;
+        template <class... Args>
+        const_reference unchecked(Args... args) const;
         template <class S>
         disable_integral_t<S, const_reference> operator[](const S& index) const;
         template <class I>
@@ -452,7 +470,7 @@ namespace xt
         bool broadcast_shape(S& shape, bool reuse_cache = false) const;
 
         template <class S>
-        bool is_trivial_broadcast(const S& strides) const noexcept;
+        bool has_linear_assign(const S& strides) const noexcept;
 
         template <class S>
         const_stepper stepper_begin(const S& shape) const noexcept;
@@ -481,14 +499,17 @@ namespace xt
         template <class F, class E, class X>
         inline auto reduce_impl(F&& f, E&& e, X&& axes, evaluation_strategy::lazy)
         {
-            using reducer_type = xreducer<F, const_xclosure_t<E>, xtl::const_closure_type_t<X>>;
-            return reducer_type(std::forward<F>(f), std::forward<E>(e), std::forward<X>(axes));
+            decltype(auto) normalized_axes = normalize_axis(e, std::forward<X>(axes));
+            using reducer_type = xreducer<F, const_xclosure_t<E>, xtl::const_closure_type_t<decltype(normalized_axes)>>;
+            return reducer_type(std::forward<F>(f), std::forward<E>(e), std::forward<decltype(normalized_axes)>(normalized_axes));
         }
 
+
         template <class F, class E, class X>
         inline auto reduce_impl(F&& f, E&& e, X&& axes, evaluation_strategy::immediate)
         {
-            return reduce_immediate(std::forward<F>(f), std::forward<E>(e), std::forward<X>(axes));
+            decltype(auto) normalized_axes = normalize_axis(e, std::forward<X>(axes));
+            return reduce_immediate(std::forward<F>(f), eval(std::forward<E>(e)), std::forward<decltype(normalized_axes)>(normalized_axes));
         }
     }
 
@@ -514,7 +535,7 @@ namespace xt
     template <class F, class E, class EVS, class>
     inline auto reduce(F&& f, E&& e, EVS evaluation_strategy)
     {
-        typename std::decay_t<E>::shape_type ar;
+        xindex_type_t<typename std::decay_t<E>::shape_type> ar;
         resize_container(ar, e.dimension());
         std::iota(ar.begin(), ar.end(), 0);
         return detail::reduce_impl(std::forward<F>(f), std::forward<E>(e), std::move(ar), evaluation_strategy);
@@ -524,16 +545,18 @@ namespace xt
     template <class F, class E, class I, class EVS>
     inline auto reduce(F&& f, E&& e, std::initializer_list<I> axes, EVS evaluation_strategy)
     {
-        using axes_type = std::vector<typename std::decay_t<E>::size_type>;
+        using axes_type = std::vector<std::size_t>;
+        auto ax = xt::forward_normalize<axes_type>(e, axes);
         using reducer_type = xreducer<F, const_xclosure_t<E>, axes_type>;
-        return detail::reduce_impl(std::forward<F>(f), std::forward<E>(e), xtl::forward_sequence<axes_type>(axes), evaluation_strategy);
+        return detail::reduce_impl(std::forward<F>(f), std::forward<E>(e), std::move(ax), evaluation_strategy);
     }
 #else
     template <class F, class E, class I, std::size_t N, class EVS>
     inline auto reduce(F&& f, E&& e, const I (&axes)[N], EVS evaluation_strategy)
     {
-        using axes_type = std::array<typename std::decay_t<E>::size_type, N>;
-        return detail::reduce_impl(std::forward<F>(f), std::forward<E>(e), xtl::forward_sequence<axes_type>(axes), evaluation_strategy);
+        using axes_type = std::array<std::size_t, N>;
+        auto ax = xt::forward_normalize<axes_type>(e, axes);
+        return detail::reduce_impl(std::forward<F>(f), std::forward<E>(e), std::move(ax), evaluation_strategy);
     }
 #endif
 
@@ -592,6 +615,49 @@ namespace xt
      * xreducer utils *
      ******************/
 
+    namespace detail
+    {
+        template <std::size_t X, std::size_t... I>
+        struct in
+        {
+            constexpr static bool value = xtl::disjunction<std::integral_constant<bool, X == I>...>::value;
+        };
+
+        template <std::size_t Z, class S1, class S2, class R>
+        struct fixed_xreducer_shape_type_impl;
+
+        template <std::size_t Z, std::size_t... I, std::size_t... J, std::size_t... R>
+        struct fixed_xreducer_shape_type_impl<Z, fixed_shape<I...>, fixed_shape<J...>, fixed_shape<R...>>
+        {
+            using type = std::conditional_t<in<Z, J...>::value,
+                                            typename fixed_xreducer_shape_type_impl<Z - 1, fixed_shape<I...>, fixed_shape<J...>,
+                                                                                    fixed_shape<R...>>::type,
+                                            typename fixed_xreducer_shape_type_impl<Z - 1, fixed_shape<I...>, fixed_shape<J...>,
+                                                                                    fixed_shape<detail::at<Z, I...>::value, R...>>::type>;
+        };
+
+        template <std::size_t... I, std::size_t... J, std::size_t... R>
+        struct fixed_xreducer_shape_type_impl<0, fixed_shape<I...>, fixed_shape<J...>, fixed_shape<R...>>
+        {
+            using type = std::conditional_t<in<0, J...>::value,
+                                            fixed_shape<R...>,
+                                            fixed_shape<detail::at<0, I...>::value, R...>>;
+        };
+    }
+
+    template <class S1, class S2>
+    struct fixed_xreducer_shape_type;
+
+    template <std::size_t... I, std::size_t... J>
+    struct fixed_xreducer_shape_type<fixed_shape<I...>, fixed_shape<J...>>
+    {
+        using type = typename detail::fixed_xreducer_shape_type_impl<sizeof...(I) - 1, 
+                                                                    fixed_shape<I...>,
+                                                                    fixed_shape<J...>,
+                                                                    fixed_shape<>>::type;
+    };
+
+
     // meta-function returning the shape type for an xreducer
     template <class ST, class X>
     struct xreducer_shape_type
@@ -605,6 +671,19 @@ namespace xt
         using type = std::array<I2, N1 - N2>;
     };
 
+    template <std::size_t... I, class I2, std::size_t N2>
+    struct xreducer_shape_type<fixed_shape<I...>, std::array<I2, N2>>
+    {
+        using type = std::array<I2, sizeof...(I) - N2>;
+    };
+
+    // Note adding "A" to prevent compilation in case nothing else matches 
+    template <std::size_t... I, std::size_t... J>
+    struct xreducer_shape_type<fixed_shape<I...>, fixed_shape<J...>>
+    {
+        using type = std::array<std::size_t, sizeof...(I) - sizeof...(J)>;
+    };
+
     namespace detail
     {
         template <class InputIt, class ExcludeIt, class OutputIt>
@@ -752,6 +831,34 @@ namespace xt
         check_access(shape(), static_cast<size_type>(args)...);
         return this->operator()(args...);
     }
+
+    /**
+     * Returns a constant reference to the element at the specified position in the reducer.
+     * @param args a list of indices specifying the position in the reducer. Indices
+     * must be unsigned integers, the number of indices must be equal to the number of
+     * dimensions of the reducer, else the behavior is undefined.
+     *
+     * @warning This method is meant for performance, for expressions with a dynamic
+     * number of dimensions (i.e. not known at compile time). Since it may have
+     * undefined behavior (see parameters), operator() should be prefered whenever
+     * it is possible.
+     * @warning This method is NOT compatible with broadcasting, meaning the following
+     * code has undefined behavior:
+     * \code{.cpp}
+     * xt::xarray<double> a = {{0, 1}, {2, 3}};
+     * xt::xarray<double> b = {0, 1};
+     * auto fd = a + b;
+     * double res = fd.uncheked(0, 1);
+     * \endcode
+     */
+    template <class F, class CT, class X>
+    template <class... Args>
+    inline auto xreducer<F, CT, X>::unchecked(Args... args) const -> const_reference
+    {
+        std::array<std::size_t, sizeof...(Args)> arg_array = { { static_cast<std::size_t>(args)... } };
+        return element(arg_array.cbegin(), arg_array.cend());
+    }
+
     /**
      * Returns a constant reference to the element at the specified position in the reducer.
      * @param index a sequence of indices specifying the position in the reducer. Indices
@@ -795,7 +902,6 @@ namespace xt
         auto stepper = const_stepper(*this, 0);
         size_type dim = 0;
         // drop left most elements
-        using difference_type = typename std::iterator_traits<It>::difference_type;
         auto size = std::ptrdiff_t(dimension()) - std::distance(first, last);
         auto begin = first - size;
         while (begin != last)
@@ -821,6 +927,7 @@ namespace xt
     /**
      * Broadcast the shape of the reducer to the specified parameter.
      * @param shape the result shape
+     * @param reuse_cache parameter for internal optimization
      * @return a boolean indicating whether the broadcasting is trivial
      */
     template <class F, class CT, class X>
@@ -831,13 +938,13 @@ namespace xt
     }
 
     /**
-     * Compares the specified strides with those of the container to see whether
-     * the broadcasting is trivial.
-     * @return a boolean indicating whether the broadcasting is trivial
-     */
+    * Checks whether the xreducer can be linearly assigned to an expression
+    * with the specified strides.
+    * @return a boolean indicating whether a linear assign is possible
+    */
     template <class F, class CT, class X>
     template <class S>
-    inline bool xreducer<F, CT, X>::is_trivial_broadcast(const S& /*strides*/) const noexcept
+    inline bool xreducer<F, CT, X>::has_linear_assign(const S& /*strides*/) const noexcept
     {
         return false;
     }
