@@ -805,7 +805,7 @@ namespace xt
     template <class T, std::size_t N, class A, bool Init>
     inline void svector<T, N, A, Init>::assign(size_type n, const value_type& v)
     {
-        if (n > N)
+        if (n > N && n > capacity())
         {
             grow(n);
         }
@@ -825,7 +825,7 @@ namespace xt
     inline void svector<T, N, A, Init>::assign(IT other_begin, IT other_end)
     {
         std::size_t size = static_cast<std::size_t>(other_end - other_begin);
-        if (size > N)
+        if (size > N && size > capacity())
         {
             grow(size);
         }
@@ -860,7 +860,7 @@ namespace xt
     template <class T, std::size_t N, class A, bool Init>
     void svector<T, N, A, Init>::resize(size_type n)
     {
-        if (n > N)
+        if (n > N && n > capacity())
         {
             grow(n);
         }
@@ -1115,7 +1115,7 @@ namespace xt
         }
 
         // Swap the shared elements.
-        size_t num_shared = (std::min)(this->size(), rhs.size());
+        std::size_t num_shared = (std::min)(this->size(), rhs.size());
 
         for (size_type i = 0; i != num_shared; ++i)
         {
@@ -1125,7 +1125,7 @@ namespace xt
         // Copy over the extra elts.
         if (this->size() > rhs.size())
         {
-            size_t elements_diff = this->size() - rhs.size();
+            std::size_t elements_diff = this->size() - rhs.size();
             std::copy(this->begin() + num_shared, this->end(), rhs.end());
             rhs.m_end = rhs.end() + elements_diff;
             this->destroy_range(this->begin() + num_shared, this->end());
@@ -1133,7 +1133,7 @@ namespace xt
         }
         else if (rhs.size() > this->size())
         {
-            size_t elements_diff = rhs.size() - this->size();
+            std::size_t elements_diff = rhs.size() - this->size();
             std::uninitialized_copy(rhs.begin() + num_shared, rhs.end(), this->end());
             this->m_end = this->end() + elements_diff;
             this->destroy_range(rhs.begin() + num_shared, rhs.end());
@@ -1161,7 +1161,9 @@ namespace xt
         else
         {
             // If this wasn't grown from the inline copy, grow the allocated space.
-            new_alloc = reinterpret_cast<pointer>(realloc(this->m_begin, new_capacity * sizeof(T)));
+            new_alloc = m_allocator.allocate(new_capacity);
+            std::uninitialized_copy(m_begin, m_end, new_alloc);
+            m_allocator.deallocate(m_begin, std::size_t(m_capacity - m_begin));
         }
         XTENSOR_ASSERT(new_alloc);
 
@@ -1225,7 +1227,14 @@ namespace xt
         lhs.swap(rhs);
     }
 
-#define XTENSOR_SELECT_ALIGN (XTENSOR_ALIGNMENT != 0 ? XTENSOR_ALIGNMENT : alignof(T))
+    #define XTENSOR_SELECT_ALIGN (XTENSOR_ALIGNMENT != 0 ? XTENSOR_ALIGNMENT : alignof(T))
+
+    template <class X, class T, std::size_t N, class A, bool B>
+    struct rebind_container<X, svector<T, N, A, B>>
+    {
+        using allocator = typename A::template rebind<X>::other;
+        using type = svector<X, N, allocator, B>;
+    };
 
     /**
      * This array class is modeled after ``std::array`` but adds optional alignment through a template parameter.
@@ -1249,6 +1258,47 @@ namespace xt
     #define XTENSOR_CONST const
 #endif
 
+#if defined(__GNUC__) && __GNUC__ < 5 && !defined(__clang__)
+    #define GCC4_FALLBACK
+
+    namespace const_array_detail
+    {
+        template <class T, std::size_t N>
+        struct array_traits
+        {
+            using storage_type = T[N];
+
+            static constexpr T& ref(const storage_type& t, std::size_t n) noexcept
+            {
+                return const_cast<T&>(t[n]);
+            }
+
+            static constexpr T* ptr(const storage_type& t) noexcept
+            {
+                return const_cast<T*>(t);
+            }
+        };
+
+        template <class T>
+        struct array_traits<T, 0>
+        {
+            struct empty {};
+
+            using storage_type = empty;
+
+            static constexpr T& ref(const storage_type& /*t*/, std::size_t /*n*/) noexcept
+            {
+                return *static_cast<T*>(nullptr);
+            }
+
+            static constexpr T* ptr(const storage_type& /*t*/) noexcept
+            {
+                return nullptr;
+            }
+        };
+    }
+#endif
+
     /**
      * A std::array like class with all member function (except reverse iterators)
      * as constexpr. The data is immutable once set.
@@ -1271,7 +1321,11 @@ namespace xt
 
         constexpr const_reference operator[](std::size_t idx) const
         {
+        #ifdef GCC4_FALLBACK
+            return const_array_detail::array_traits<T, N>::ref(m_data, idx);
+        #else
             return m_data[idx];
+        #endif
         }
 
         constexpr const_iterator begin() const noexcept
@@ -1286,12 +1340,12 @@ namespace xt
 
         constexpr const_iterator cbegin() const noexcept
         {
-            return m_data;
+            return data();
         }
 
         constexpr const_iterator cend() const noexcept
         {
-            return m_data + N;
+            return data() + N;
         }
 
         // TODO make constexpr once C++17 arrives
@@ -1317,17 +1371,30 @@ namespace xt
 
         constexpr const_pointer data() const noexcept
         {
+        #ifdef GCC4_FALLBACK
+            return const_array_detail::array_traits<T, N>::ptr(m_data);
+        #else
             return m_data;
+        #endif
         }
 
         constexpr const_reference front() const noexcept
         {
+        #ifdef GCC4_FALLBACK
+            return const_array_detail::array_traits<T, N>::ref(m_data, 0);
+        #else
             return m_data[0];
+        #endif
         }
 
         constexpr const_reference back() const noexcept
         {
+        #ifdef GCC4_FALLBACK
+            return N ? const_array_detail::array_traits<T, N>::ref(m_data, N - 1) :
+                       const_array_detail::array_traits<T, N>::ref(m_data, 0);
+        #else
             return m_data[size() - 1];
+        #endif
         }
 
         constexpr size_type size() const noexcept
@@ -1335,10 +1402,322 @@ namespace xt
             return N;
         }
 
+    #ifdef GCC4_FALLBACK
+        XTENSOR_CONST typename const_array_detail::array_traits<T, N>::storage_type m_data;
+    #else
         XTENSOR_CONST T m_data[N > 0 ? N : 1];
+    #endif
     };
+
+#undef GCC4_FALLBACK
+
+
+// Workaround for rebind_container problems on GCC 8  with C++17 enabled
+#if defined(__GNUC__) && __GNUC__ > 6 && !defined(__clang__) && __cplusplus >= 201703L
+    template <class X, class T, std::size_t N>
+    struct rebind_container<X, aligned_array<T, N>>
+    {
+        using type = aligned_array<X, N>;
+    };
+
+    template <class X, class T, std::size_t N>
+    struct rebind_container<X, const_array<T, N>>
+    {
+        using type = const_array<X, N>;
+    };
+#endif
+
+    /**
+     * @class fixed_shape
+     * Fixed shape implementation for compile time defined arrays.
+     * @sa xshape
+     */
+    template <std::size_t... X>
+    class fixed_shape
+    {
+    public:
+
+#if defined(_MSC_VER)
+        using cast_type = std::array<std::size_t, sizeof...(X)>;
+        #define XTENSOR_FIXED_SHAPE_CONSTEXPR inline
+#else
+        using cast_type = const_array<std::size_t, sizeof...(X)>;
+        #define XTENSOR_FIXED_SHAPE_CONSTEXPR constexpr
+#endif
+        using value_type = std::size_t;
+        using size_type = std::size_t;
+
+        constexpr static std::size_t size()
+        {
+            return sizeof...(X);
+        }
+
+        XTENSOR_FIXED_SHAPE_CONSTEXPR operator cast_type() const
+        {
+            return cast_type({X...});
+        }
+
+        XTENSOR_FIXED_SHAPE_CONSTEXPR auto begin() const
+        {
+            return m_array.begin();
+        }
+
+        XTENSOR_FIXED_SHAPE_CONSTEXPR auto end() const
+        {
+            return m_array.end();
+        }
+
+        auto rbegin() const
+        {
+            return m_array.rbegin();
+        }
+
+        auto rend() const
+        {
+            return m_array.rend();
+        }
+
+        XTENSOR_FIXED_SHAPE_CONSTEXPR auto cbegin() const
+        {
+            return m_array.cbegin();
+        }
+
+        XTENSOR_FIXED_SHAPE_CONSTEXPR auto cend() const
+        {
+            return m_array.cend();
+        }
+
+        XTENSOR_FIXED_SHAPE_CONSTEXPR std::size_t operator[](std::size_t idx) const
+        {
+            return m_array[idx];
+        }
+
+    private:
+
+         XTENSOR_CONSTEXPR_ENHANCED_STATIC cast_type m_array = cast_type({X...});
+    };
+
+#ifdef XTENSOR_HAS_CONSTEXPR_ENHANCED
+    template <std::size_t... X>
+    constexpr typename fixed_shape<X...>::cast_type fixed_shape<X...>::m_array;
+#endif
+
+#undef XTENSOR_FIXED_SHAPE_CONSTEXPR
+
+    template <class E, std::ptrdiff_t Start, std::ptrdiff_t End = -1>
+    class sequence_view
+    {
+    public:
+
+        using value_type = typename E::value_type;
+        using reference = typename E::reference;
+        using const_reference = typename E::const_reference;
+        using pointer = typename E::pointer;
+        using const_pointer = typename E::const_pointer;
+
+        using size_type = typename E::size_type;
+        using difference_type = typename E::difference_type;
+
+        using iterator = typename E::iterator;
+        using const_iterator = typename E::const_iterator;
+        using reverse_iterator = typename E::reverse_iterator;
+        using const_reverse_iterator = typename E::const_reverse_iterator;
+
+        explicit sequence_view(const E& container);
+
+        template <std::ptrdiff_t OS, std::ptrdiff_t OE>
+        explicit sequence_view(const sequence_view<E, OS, OE>& other);
+
+        size_type size() const;
+        const_reference operator[](std::size_t idx) const;
+
+        const_iterator end() const;
+        const_iterator begin() const;
+        const_iterator cend() const;
+        const_iterator cbegin() const;
+
+        const_reverse_iterator rend() const;
+        const_reverse_iterator rbegin() const;
+        const_reverse_iterator crend() const;
+        const_reverse_iterator crbegin() const;
+
+        const_reference front() const;
+        const_reference back() const;
+
+        const E& storage() const;
+
+    private:
+        const E& m_sequence;
+    };
+
+    template <class E, std::ptrdiff_t Start, std::ptrdiff_t End>
+    sequence_view<E, Start, End>::sequence_view(const E& container)
+        : m_sequence(container)
+    {
+    }
+
+    template <class E, std::ptrdiff_t Start, std::ptrdiff_t End>
+    template <std::ptrdiff_t OS, std::ptrdiff_t OE>
+    sequence_view<E, Start, End>::sequence_view(const sequence_view<E, OS, OE>& other)
+        : m_sequence(other.storage())
+    {
+    }
+
+    template <class E, std::ptrdiff_t Start, std::ptrdiff_t End>
+    auto sequence_view<E, Start, End>::size() const -> size_type
+    {
+        if (End == -1)
+        {
+            return m_sequence.size() - static_cast<size_type>(Start);
+        }
+        else
+        {
+            return static_cast<size_type>(End - Start);
+        }
+    }
+
+    template <class E, std::ptrdiff_t Start, std::ptrdiff_t End>
+    auto sequence_view<E, Start, End>::operator[](std::size_t idx) const -> const_reference
+    {
+        return m_sequence[idx + static_cast<std::size_t>(Start)];
+    }
+
+    template <class E, std::ptrdiff_t Start, std::ptrdiff_t End>
+    auto  sequence_view<E, Start, End>::end() const -> const_iterator
+    {
+        if (End != -1)
+        {
+            return m_sequence.begin() + End;
+        }
+        else
+        {
+            return m_sequence.end();
+        }
+    }
+
+    template <class E, std::ptrdiff_t Start, std::ptrdiff_t End>
+    auto  sequence_view<E, Start, End>::begin() const -> const_iterator
+    {
+        return m_sequence.begin() + Start;
+    }
+
+    template <class E, std::ptrdiff_t Start, std::ptrdiff_t End>
+    auto  sequence_view<E, Start, End>::cend() const -> const_iterator
+    {
+        return end();
+    }
+
+    template <class E, std::ptrdiff_t Start, std::ptrdiff_t End>
+    auto  sequence_view<E, Start, End>::cbegin() const -> const_iterator
+    {
+        return begin();
+    }
+
+    template <class E, std::ptrdiff_t Start, std::ptrdiff_t End>
+    auto sequence_view<E, Start, End>::rend() const -> const_reverse_iterator
+    {
+        return const_reverse_iterator(begin());
+    }
+
+    template <class E, std::ptrdiff_t Start, std::ptrdiff_t End>
+    auto sequence_view<E, Start, End>::rbegin() const -> const_reverse_iterator
+    {
+        return const_reverse_iterator(end());
+    }
+
+    template <class E, std::ptrdiff_t Start, std::ptrdiff_t End>
+    auto sequence_view<E, Start, End>::crend() const -> const_reverse_iterator
+    {
+        return rend();
+    }
+
+    template <class E, std::ptrdiff_t Start, std::ptrdiff_t End>
+    auto sequence_view<E, Start, End>::crbegin() const -> const_reverse_iterator
+    {
+        return rbegin();
+    }
+
+    template <class E, std::ptrdiff_t Start, std::ptrdiff_t End>
+    auto sequence_view<E, Start, End>::front() const -> const_reference
+    {
+        return *(m_sequence.begin() + Start);
+    }
+
+    template <class E, std::ptrdiff_t Start, std::ptrdiff_t End>
+    auto sequence_view<E, Start, End>::back() const -> const_reference
+    {
+        if (End == -1)
+        {
+            return m_sequence.back();
+        }
+        else
+        {
+            return m_sequence[static_cast<std::size_t>(End - 1)];
+        }
+    }
+
+    template <class E, std::ptrdiff_t Start, std::ptrdiff_t End>
+    const E& sequence_view<E, Start, End>::storage() const
+    {
+        return m_sequence;
+    }
+
+
+    template <class T, std::ptrdiff_t TB, std::ptrdiff_t TE>
+    inline bool operator==(const sequence_view<T, TB, TE>& lhs, const sequence_view<T, TB, TE>& rhs)
+    {
+        return lhs.size() == rhs.size() && std::equal(lhs.begin(), lhs.end(), rhs.begin());
+    }
+
+    template <class T, std::ptrdiff_t TB, std::ptrdiff_t TE>
+    inline bool operator!=(const sequence_view<T, TB, TE>& lhs, const sequence_view<T, TB, TE>& rhs)
+    {
+        return !(lhs == rhs);
+    }
 }
 
+/******************************
+ * std::tuple_size extensions *
+ ******************************/
+
+// The C++ standard defines tuple_size as a class, however
+// G++ 8 C++ library does define it as a struct hence we get
+// clang warnings here
+
+#if defined(__clang__)
+    #pragma clang diagnostic push
+    #pragma clang diagnostic ignored "-Wmismatched-tags"
+#endif
+
+namespace std
+{
+    template <class T, std::size_t N>
+    class tuple_size<xt::const_array<T, N>> :
+        public integral_constant<std::size_t, N>
+    {
+    };
+
+    template <std::size_t... N>
+    class tuple_size<xt::fixed_shape<N...>> :
+        public integral_constant<std::size_t, sizeof...(N)>
+    {
+    };
+
+    template <class T, std::ptrdiff_t Start, std::ptrdiff_t End>
+    class tuple_size<xt::sequence_view<T, Start, End>> :
+        public integral_constant<std::size_t, End - Start>
+    {
+    };
+
+    // Undefine tuple size for not-known sequence view size
+    template <class T, std::ptrdiff_t Start>
+    class tuple_size<xt::sequence_view<T, Start, -1>>;
+}
+
+#if defined(__clang__)
+    #pragma clang diagnostic pop
+#endif
+
 #undef XTENSOR_CONST
 #undef XTENSOR_ALIGNMENT
 #undef XTENSOR_SELECT_ALIGN
