@@ -14,6 +14,10 @@
 #include <type_traits>
 #include <utility>
 
+#ifdef __CLING__
+#include <nlohmann/json.hpp>
+#endif
+
 #include "xclosure.hpp"
 #include "xmeta_utils.hpp"
 #include "xtl_config.hpp"
@@ -109,7 +113,7 @@ namespace xtl
 
         template <class T1, class B1, class T2>
         struct common_optional_impl<xoptional<T1, B1>, T2>
-            : common_optional_impl<T2, xoptional<T1, B1>>
+            : common_optional_impl<T1, T2>
         {
         };
 
@@ -782,7 +786,7 @@ namespace xtl
     template <class CT, class CB>
     void xoptional<CT, CB>::swap(xoptional& other)
     {
-        std::swap(m_value, other.m_flag);
+        std::swap(m_value, other.m_value);
         std::swap(m_flag, other.m_flag);
     }
 
@@ -834,6 +838,18 @@ namespace xtl
         return out;
     }
 
+#ifdef __CLING__
+    template <class T, class B>
+    nlohmann::json mime_bundle_repr(const xoptional<T, B>& v)
+    {
+        auto bundle = nlohmann::json::object();
+        std::stringstream tmp;
+        tmp << v;
+        bundle["text/plain"] = tmp.str();
+        return bundle;
+    }
+#endif
+
     template <class T1, class B1, class T2, class B2>
     inline auto operator==(const xoptional<T1, B1>& e1, const xoptional<T2, B2>& e2) noexcept
         -> bool
